@@ -4,61 +4,45 @@ import base64
 import datetime
 import hashlib
 import hmac
-import json
 import time
 from dataclasses import dataclass
-from typing import Any, Callable
+from typing import TYPE_CHECKING, Any, Callable
+from urllib.parse import urlencode
 
-import aiohttp
 from aiohttp.formdata import FormData
 from aiohttp.hdrs import METH_DELETE, METH_GET
 from aiohttp.payload import JsonPayload
 from multidict import CIMultiDict, MultiDict
 from yarl import URL
 
+if TYPE_CHECKING:
+    import aiohttp
+
 
 class Auth:
     @staticmethod
     def bybit(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
-        method: str = args[0]
         url: URL = args[1]
         data: dict[str, Any] = kwargs["data"] or {}
+        headers: CIMultiDict = kwargs["headers"]
 
         session: aiohttp.ClientSession = kwargs["session"]
         key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
         secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
 
-        if url.scheme == "https":
-            expires = str(int((time.time() - 5.0) * 1000))
-            recv_window = (
-                "recv_window" if not url.path.startswith("/spot") else "recvWindow"
-            )
-            auth_params = {"api_key": key, "timestamp": expires, recv_window: 10000}
-            if method in (METH_GET, METH_DELETE):
-                query = MultiDict(url.query)
-                query.extend(auth_params)
-                query_string = "&".join(f"{k}={v}" for k, v in sorted(query.items()))
-                sign = hmac.new(
-                    secret, query_string.encode(), hashlib.sha256
-                ).hexdigest()
-                query.extend({"sign": sign})
-                url = url.with_query(query)
-                args = (method, url)
-            else:
-                data.update(auth_params)
-                body = FormData(sorted(data.items()))()
-                sign = hmac.new(secret, body._value, hashlib.sha256).hexdigest()
-                body._value += f"&sign={sign}".encode()
-                body._size = len(body._value)
-                kwargs.update({"data": body})
-        elif url.scheme == "wss":
-            query = MultiDict(url.query)
-            expires = str(int((time.time() + 5.0) * 1000))
-            path = f"{method}/realtime{expires}"
-            signature = hmac.new(secret, path.encode(), hashlib.sha256).hexdigest()
-            query.extend({"api_key": key, "expires": expires, "signature": signature})
-            url = url.with_query(query)
-            args = (method, url)
+        timestamp = str(int(time.time() * 1000))
+        query_string = url.raw_query_string
+        body = JsonPayload(data) if data else FormData(data)()
+        text = f"{timestamp}{key}{query_string}".encode() + body._value
+        signature = hmac.new(secret, text, hashlib.sha256).hexdigest()
+        kwargs.update({"data": body})
+        headers.update(
+            {
+                "X-BAPI-API-KEY": key,
+                "X-BAPI-TIMESTAMP": timestamp,
+                "X-BAPI-SIGN": signature,
+            }
+        )
 
         return args
 
@@ -73,36 +57,36 @@ class Auth:
         key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
         secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
 
-        expires = str(int(time.time() * 1000))
-        if method == METH_GET:
-            if url.scheme == "https":
-                query = MultiDict(url.query)
-                query.extend({"timestamp": expires})
-                query_string = "&".join(f"{k}={v}" for k, v in query.items())
-                signature = hmac.new(
-                    secret, query_string.encode(), hashlib.sha256
-                ).hexdigest()
-                query.extend({"signature": signature})
-                url = url.with_query(query)
-                args = (
-                    method,
-                    url,
-                )
-        else:
-            data.update({"timestamp": expires})
-            # patch (issue #190, #192)
-            if url.path != "/api/v3/userDataStream":
-                body = FormData(data)()
-            else:
-                body = FormData()()
-            signature = hmac.new(secret, body._value, hashlib.sha256).hexdigest()
-            # patch (issue #190, #192)
-            if url.path != "/api/v3/userDataStream":
-                body._value += f"&signature={signature}".encode()
-            body._size = len(body._value)
-            kwargs.update({"data": body})
+        # Do not sign WebSocket upgrade requests
+        if headers.get("Upgrade") == "websocket":
+            args = (method, url)
+            return args
+
         headers.update({"X-MBX-APIKEY": key})
 
+        # NOTE: Security Type `USER_STREAM` patch (Issue #192), Only edit header
+        if url.name in {"userDataStream", "listen-key", "listenKey"}:
+            args = (method, url)
+            return args
+
+        expires = str(int(time.time() * 1000))
+        query = MultiDict(url.query)
+        body = FormData(data)()
+
+        query.extend({"timestamp": expires})
+        url = url.with_query(query)
+        query = MultiDict(url.query)
+
+        query_string = url.raw_query_string.encode()
+        signature = hmac.new(
+            secret, query_string + body._value, hashlib.sha256
+        ).hexdigest()
+
+        query.extend({"signature": signature})
+        url = url.with_query(query)
+        args = (method, url)
+        kwargs.update({"data": body})
+
         return args
 
     @staticmethod
@@ -155,42 +139,6 @@ class Auth:
 
         return args
 
-    @staticmethod
-    def liquid(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
-        url: URL = args[1]
-        data: dict[str, Any] = kwargs["data"] or {}
-        headers: CIMultiDict = kwargs["headers"]
-
-        session: aiohttp.ClientSession = kwargs["session"]
-        key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
-        secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
-
-        json_payload = json.dumps(
-            {
-                "path": url.raw_path_qs,
-                "nonce": str(int(time.time() * 1000)),
-                "token_id": key,
-            },
-            separators=(",", ":"),
-        ).encode()
-        json_header = json.dumps(
-            {"typ": "JWT", "alg": "HS256"},
-            separators=(",", ":"),
-        ).encode()
-        segments = [
-            base64.urlsafe_b64encode(json_header).replace(b"=", b""),
-            base64.urlsafe_b64encode(json_payload).replace(b"=", b""),
-        ]
-        signing_input = b".".join(segments)
-        signature = hmac.new(secret, signing_input, hashlib.sha256).digest()
-        segments.append(base64.urlsafe_b64encode(signature).replace(b"=", b""))
-        encoded_string = b".".join(segments).decode()
-        body = JsonPayload(data) if data else FormData(data)()
-        kwargs.update({"data": body})
-        headers.update({"X-Quoine-API-Version": "2", "X-Quoine-Auth": encoded_string})
-
-        return args
-
     @staticmethod
     def bitbank(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
         method: str = args[0]
@@ -205,39 +153,23 @@ class Auth:
         path = url.raw_path_qs
         body = JsonPayload(data) if data else FormData(data)()
         nonce = str(int(time.time() * 1000))
+        window = headers.get("ACCESS-TIME-WINDOW", "")
         if method == METH_GET:
-            text = f"{nonce}{path}".encode()
+            text = f"{nonce}{window}{path}".encode()
         else:
-            text = nonce.encode() + body._value
+            text = f"{nonce}{window}".encode() + body._value
         signature = hmac.new(secret, text, hashlib.sha256).hexdigest()
         kwargs.update({"data": body})
         headers.update(
-            {"ACCESS-KEY": key, "ACCESS-NONCE": nonce, "ACCESS-SIGNATURE": signature}
+            {
+                "ACCESS-KEY": key,
+                "ACCESS-REQUEST-TIME": nonce,
+                "ACCESS-SIGNATURE": signature,
+            }
         )
 
         return args
 
-    @staticmethod
-    def ftx(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
-        method: str = args[0]
-        url: URL = args[1]
-        data: dict[str, Any] = kwargs["data"] or {}
-        headers: CIMultiDict = kwargs["headers"]
-
-        session: aiohttp.ClientSession = kwargs["session"]
-        key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
-        secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
-
-        path = url.raw_path_qs
-        body = JsonPayload(data) if data else FormData(data)()
-        ts = str(int(time.time() * 1000))
-        text = f"{ts}{method}{path}".encode() + body._value
-        signature = hmac.new(secret, text, hashlib.sha256).hexdigest()
-        kwargs.update({"data": body})
-        headers.update({"FTX-KEY": key, "FTX-SIGN": signature, "FTX-TS": ts})
-
-        return args
-
     @staticmethod
     def bitmex(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
         method: str = args[0]
@@ -317,7 +249,7 @@ class Auth:
         headers: CIMultiDict = kwargs["headers"]
 
         session: aiohttp.ClientSession = kwargs["session"]
-        api_name = NameSelector.okx(headers)
+        api_name = DynamicNameSelector.okx(args, kwargs)
         key: str = session.__dict__["_apis"][api_name][0]
         secret: bytes = session.__dict__["_apis"][api_name][1]
         passphrase: str = session.__dict__["_apis"][api_name][2]
@@ -421,23 +353,16 @@ class Auth:
         query = MultiDict(url.query)
         body = FormData(data)()
 
-        if query:
-            query.extend({"timestamp": timestamp})
-            url = url.with_query(query)
-            query = MultiDict(url.query)
-        else:
-            body._value += f"&timestamp={timestamp}".encode()
+        query.extend({"timestamp": timestamp})
+        url = url.with_query(query)
+        query = MultiDict(url.query)
 
         query_string = url.raw_query_string.encode()
         signature = hmac.new(
             secret, query_string + body._value, hashlib.sha256
         ).hexdigest()
 
-        if query:
-            query.extend({"signature": signature})
-        else:
-            body._value += f"&signature={signature}".encode()
-            body._size += len(body._value)
+        query.extend({"signature": signature})
 
         url = url.with_query(query)
         args = (method, url)
@@ -459,16 +384,14 @@ class Auth:
         passphrase: str = session.__dict__["_apis"][Hosts.items[url.host].name][2]
 
         now = int(time.time() * 1000)
-        if method == "GET":
-            str_to_sign = str(now) + method + url.path_qs
-        else:
-            body = JsonPayload(data) if data else FormData(data)()
-            headers["Content-Type"] = "application/json"
-            kwargs.update({"data": body._value})
-            str_to_sign = str(now) + method + url.path + body._value.decode()
+        body = JsonPayload(data) if data else FormData(data)()
+        if body._value:
+            kwargs.update({"data": body})
+
+        str_to_sign = f"{now}{method}{url.path_qs}".encode() + body._value
 
         signature = base64.b64encode(
-            hmac.new(secret, str_to_sign.encode("utf-8"), hashlib.sha256).digest()
+            hmac.new(secret, str_to_sign, hashlib.sha256).digest()
         ).decode()
         passphrase = base64.b64encode(
             hmac.new(secret, passphrase.encode("utf-8"), hashlib.sha256).digest()
@@ -484,16 +407,89 @@ class Auth:
         )
         return args
 
+    @staticmethod
+    def okj(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
+        method: str = args[0]
+        url: URL = args[1]
+        data: dict[str, Any] = kwargs["data"] or {}
+        headers: CIMultiDict = kwargs["headers"]
+
+        session: aiohttp.ClientSession = kwargs["session"]
+        key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
+        secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
+        passphrase: str = session.__dict__["_apis"][Hosts.items[url.host].name][2]
+
+        timestamp = f'{datetime.datetime.utcnow().isoformat(timespec="milliseconds")}Z'
+        body = JsonPayload(data) if data else FormData(data)()
+        text = f"{timestamp}{method}{url.raw_path_qs}".encode() + body._value
+        sign = base64.b64encode(
+            hmac.new(secret, text, hashlib.sha256).digest()
+        ).decode()
+        kwargs.update({"data": body})
+        headers.update(
+            {
+                "OK-ACCESS-KEY": key,
+                "OK-ACCESS-SIGN": sign,
+                "OK-ACCESS-TIMESTAMP": timestamp,
+                "OK-ACCESS-PASSPHRASE": passphrase,
+                "Content-Type": "application/json",
+            }
+        )
+
+        return args
+
+    @staticmethod
+    def bittrade(args: tuple[str, URL], kwargs: dict[str, Any]) -> tuple[str, URL]:
+        method, url = args
+        data = kwargs["data"]
+
+        session: aiohttp.ClientSession = kwargs["session"]
+        key: str = session.__dict__["_apis"][Hosts.items[url.host].name][0]
+        secret: bytes = session.__dict__["_apis"][Hosts.items[url.host].name][1]
+
+        timestamp = datetime.datetime.now(datetime.timezone.utc).strftime(
+            "%Y-%m-%dT%H:%M:%S"
+        )
+
+        query = MultiDict(url.query)
+        query.extend(
+            {
+                "AccessKeyId": key,
+                "SignatureMethod": "HmacSHA256",
+                "SignatureVersion": "2",
+                "Timestamp": timestamp,
+            }
+        )
+        sorted_query = MultiDict(sorted(query.items()))
+        # NOTE: yarl.URL.raw_query_string does not encode colons(:), so use urllib.parse.urlencode instead
+        sorted_raw_query_string = urlencode(sorted_query)
+        sign_str = (
+            f"{method}\n{url.host}\n{url.raw_path}\n{sorted_raw_query_string}".encode()
+        )
+        signature = base64.b64encode(
+            hmac.new(secret, sign_str, hashlib.sha256).digest()
+        ).decode()
+        query.extend({"Signature": signature})
+
+        url = url.with_query(query)
+
+        if data:
+            kwargs.update({"data": JsonPayload(data)})
+
+        return (method, url)
+
 
 @dataclass
 class Item:
-    name: str | Callable[[CIMultiDict], str]
+    name: str | Callable[[tuple[str, URL], dict[str, Any]], str]
     func: Any
 
 
-class NameSelector:
+class DynamicNameSelector:
     @staticmethod
-    def okx(headers: CIMultiDict) -> str:
+    def okx(args: tuple[str, URL], kwargs: dict[str, Any]) -> str:
+        headers: CIMultiDict = kwargs["headers"]
+
         if "x-simulated-trading" in headers:
             if headers["x-simulated-trading"] == "1":
                 return "okx_demo"
@@ -501,46 +497,43 @@ class NameSelector:
 
 
 class Hosts:
-    items = {
+    # NOTE: yarl.URL.host is also allowed to be None. So, for brevity, relax the type check on the `items` key.
+    items: dict[str | None, Item] = {
         "api.bybit.com": Item("bybit", Auth.bybit),
         "api.bytick.com": Item("bybit", Auth.bybit),
-        "stream.bybit.com": Item("bybit", Auth.bybit),
-        "stream.bytick.com": Item("bybit", Auth.bybit),
+        "api-demo.bybit.com": Item("bybit_demo", Auth.bybit),
         "api-testnet.bybit.com": Item("bybit_testnet", Auth.bybit),
-        "stream-testnet.bybit.com": Item("bybit_testnet", Auth.bybit),
         "api.binance.com": Item("binance", Auth.binance),
+        "api-gcp.binance.com": Item("binance", Auth.binance),
         "api1.binance.com": Item("binance", Auth.binance),
         "api2.binance.com": Item("binance", Auth.binance),
         "api3.binance.com": Item("binance", Auth.binance),
-        "stream.binance.com": Item("binance", Auth.binance),
+        "api4.binance.com": Item("binance", Auth.binance),
+        "testnet.binance.vision": Item("binancespot_testnet", Auth.binance),
         "fapi.binance.com": Item("binance", Auth.binance),
-        "fstream.binance.com": Item("binance", Auth.binance),
-        "fstream-auth.binance.com": Item("binance", Auth.binance),
         "dapi.binance.com": Item("binance", Auth.binance),
-        "dstream.binance.com": Item("binance", Auth.binance),
-        "vapi.binance.com": Item("binance", Auth.binance),
-        "vstream.binance.com": Item("binance", Auth.binance),
-        "testnet.binancefuture.com": Item("binance_testnet", Auth.binance),
-        "stream.binancefuture.com": Item("binance_testnet", Auth.binance),
-        "dstream.binancefuture.com": Item("binance_testnet", Auth.binance),
-        "testnet.binanceops.com": Item("binance_testnet", Auth.binance),
-        "testnetws.binanceops.com": Item("binance_testnet", Auth.binance),
+        "testnet.binancefuture.com": Item("binancefuture_testnet", Auth.binance),
         "api.bitflyer.com": Item("bitflyer", Auth.bitflyer),
         "api.coin.z.com": Item("gmocoin", Auth.gmocoin),
-        "api.liquid.com": Item("liquid", Auth.liquid),
         "api.bitbank.cc": Item("bitbank", Auth.bitbank),
-        "ftx.com": Item("ftx", Auth.ftx),
         "www.bitmex.com": Item("bitmex", Auth.bitmex),
         "testnet.bitmex.com": Item("bitmex_testnet", Auth.bitmex),
         "api.phemex.com": Item("phemex", Auth.phemex),
+        "vapi.phemex.com": Item("phemex", Auth.phemex),
         "testnet-api.phemex.com": Item("phemex_testnet", Auth.phemex),
         "coincheck.com": Item("coincheck", Auth.coincheck),
-        "www.okx.com": Item(NameSelector.okx, Auth.okx),
-        "aws.okx.com": Item(NameSelector.okx, Auth.okx),
+        "www.okx.com": Item(DynamicNameSelector.okx, Auth.okx),
+        "aws.okx.com": Item(DynamicNameSelector.okx, Auth.okx),
         "api.bitget.com": Item("bitget", Auth.bitget),
         "www.mexc.com": Item("mexc", Auth.mexc_v2),
         "contract.mexc.com": Item("mexc", Auth.mexc_v2),
         "api.mexc.com": Item("mexc", Auth.mexc_v3),
-        "api.kucoin.com": Item("kucoinspot", Auth.kucoin),
-        "api-futures.kucoin.com": Item("kucoinfuture", Auth.kucoin),
+        "api.kucoin.com": Item("kucoin", Auth.kucoin),
+        "api-futures.kucoin.com": Item("kucoin", Auth.kucoin),
+        "www.okcoin.jp": Item("okj", Auth.okj),
+        "api-cloud.bittrade.co.jp": Item("bittrade", Auth.bittrade),
     }
+
+
+class PassphraseRequiredExchanges:
+    items = {"okx", "bitget", "kucoin", "okj"}
