@@ -2,65 +2,72 @@ from __future__ import annotations
 
 import asyncio
 import logging
-from collections import deque
-from typing import Any, Awaitable, Optional, Union
+from collections import defaultdict, deque
+from typing import TYPE_CHECKING, Any, Awaitable
 
 import aiohttp
 
 from ..auth import Auth
-from ..store import DataStore, DataStoreManager
-from ..typedefs import Item
-from ..ws import ClientWebSocketResponse
+from ..store import DataStore, DataStoreCollection
+
+if TYPE_CHECKING:
+    from yarl import URL
+
+    from ..typedefs import Item
+    from ..ws import ClientWebSocketResponse
 
 logger = logging.getLogger(__name__)
 
 
-class BinanceDataStoreBase(DataStoreManager):
-    _ORDERBOOK_INIT_ENDPOINT = None
-    _ORDER_INIT_ENDPOINT = None
-    _LISTENKEY_INIT_ENDPOINT = None
-    _KLINE_INIT_ENDPOINT = None
+class BinanceDataStoreBase(DataStoreCollection):
+    """Binance の DataStoreCollection ベースクラス"""
+
+    _ORDERBOOK_INIT_ENDPOINT: str
+    _ORDER_INIT_ENDPOINT: str
+    _LISTENKEY_INIT_ENDPOINT: str
+    _KLINE_INIT_ENDPOINT: str | tuple[str, ...]
 
     def _init(self) -> None:
-        self.create("trade", datastore_class=Trade)
-        self.create("kline", datastore_class=Kline)
-        self.create("ticker", datastore_class=Ticker)
-        self.create("bookticker", datastore_class=BookTicker)
-        self.create("orderbook", datastore_class=OrderBook)
-        self.create("order", datastore_class=Order)
-        self.listenkey: Optional[str] = None
+        self._create("trade", datastore_class=Trade)
+        self._create("kline", datastore_class=Kline)
+        self._create("ticker", datastore_class=Ticker)
+        self._create("bookticker", datastore_class=BookTicker)
+        self._create("orderbook", datastore_class=OrderBook)
+        self._create("order", datastore_class=Order)
+        self.listenkey: str | None = None
 
     async def initialize(self, *aws: Awaitable[aiohttp.ClientResponse]) -> None:
-        """
+        """Initialize DataStore from HTTP response data.
+
         対応エンドポイント
 
         共通
 
-        - GET /api/v3/depth, /fapi/v1/depth, /dapi/v1/depth (DataStore: orderbook)
+        - GET /api/v3/depth, /fapi/v1/depth, /dapi/v1/depth (:attr:`.BinanceDataStoreBase.orderbook`)
 
             - Binance APIドキュメントに従ってWebSocket接続後にinitializeすること。
             - orderbook データストアの initialized がTrueになる。
 
-        - GET /api/v3/openOrders, /fapi/v1/openOrders, /dapi/v1/openOrders (DataStore: order)
-        - POST /api/v3/userDataStream, /fapi/v1/listenKey, /dapi/v1/listenKey (Property: listenkey)
+        - GET /api/v3/openOrders, /fapi/v1/openOrders, /dapi/v1/openOrders (:attr:`.BinanceDataStoreBase.order`)
+        - POST /api/v3/userDataStream, /fapi/v1/listenKey, /dapi/v1/listenKey (:attr:`.BinanceDataStoreBase.listenkey`)
             - プロパティ listenkey にlistenKeyが格納され30分ごとに PUT リクエストがスケジュールされる。
-        - GET /api/v3/klines, /fapi/v1/klines, /dapi/v3/klines, /dapi/v1/indexPriceKlines, /dapi/v1/markPriceKlines (DataStore: kline)
+        - GET /api/v3/klines, /fapi/v1/klines, /dapi/v3/klines, /dapi/v1/indexPriceKlines, /dapi/v1/markPriceKlines (:attr:`.BinanceDataStoreBase.kline`)
 
         現物
 
-        - GET /api/v3/account (DataStore: account)
-        - GET /api/v3/openOrderList (DataStore: ocoorder)
+        - GET /api/v3/account (DataStore: account)(:attr:`.BinanceSpotDataStore.account`)
+        - GET /api/v3/openOrderList (DataStore: ocoorder)(:attr:`.BinanceSpotDataStore.ocoorder`)
 
         先物
 
-        - GET /fapi/v2/balance, /dapi/v1/balance (DataStore: balance)
-        - GET /fapi/v2/positionRisk, /dapi/v1/positionRisk (DataStore: position)
+        - GET /fapi/v2/balance, /dapi/v1/balance (:attr:`.BinanceCOINMDataStore.balance`)
+        - GET /fapi/v2/positionRisk, /dapi/v1/positionRisk (:attr:`.BinanceCOINMDataStore.position`)
 
         USDⓈ-M
-        - GET /fapi/v1/indexInfo (DataStore: compositeindex)
+        - GET /fapi/v1/indexInfo (:attr:`.BinanceUSDSMDataStore.compositeindex`)
 
 
-        """  # noqa
+        """
         for f in asyncio.as_completed(aws):
             resp = await f
             data = await resp.json()
@@ -80,7 +87,7 @@ class BinanceDataStoreBase(DataStoreManager):
         """子クラス用initialize hook"""
         ...
 
-    def _is_target_endpoint(self, target: Union[str, tuple[str], None], endpoint: str):
+    def _is_target_endpoint(self, target: str | tuple[str, ...] | None, endpoint: str):
         if target:
             if isinstance(target, str):
                 return endpoint == target
@@ -166,51 +173,119 @@ class BinanceDataStoreBase(DataStoreManager):
     def _is_order_msg(self, msg: Any, event: str):
         raise NotImplementedError
 
-    @staticmethod
-    async def _listenkey(url, session: aiohttp.ClientSession):
+    async def _listenkey(self, url: URL, session: aiohttp.ClientSession):
+        if url.path.startswith(BinanceSpotDataStore._LISTENKEY_INIT_ENDPOINT):
+            params = {"listenKey": self.listenkey}
+        else:
+            params = None
         while not session.closed:
-            await session.put(url, auth=Auth)
+            async with session.put(url, params=params, auth=Auth) as resp:
+                text = await resp.text()
+            try:
+                resp.raise_for_status()
+            except aiohttp.ClientResponseError:
+                logger.error(text)
+                raise
             await asyncio.sleep(1800.0)  # 30 minutes
 
     @property
     def trade(self) -> "Trade":
-        return self.get("trade", Trade)
+        """trade/aggTrade stream.
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#aggregate-trade-streams
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#trade-streams
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Aggregate-Trade-Streams
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Aggregate-Trade-Streams
+        """
+        return self._get("trade", Trade)
 
     @property
     def kline(self) -> "Kline":
-        return self.get("kline", Kline)
+        """kline stream.
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#klinecandlestick-streams-for-utc
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Kline-Candlestick-Streams
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Kline-Candlestick-Streams
+        """
+        return self._get("kline", Kline)
 
     @property
     def ticker(self) -> "Ticker":
-        return self.get("ticker", Ticker)
+        """24hrMiniTicker/24hrTicker stream.
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#individual-symbol-mini-ticker-stream
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#individual-symbol-ticker-streams
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Individual-Symbol-Mini-Ticker-Stream
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Individual-Symbol-Ticker-Streams
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Individual-Symbol-Mini-Ticker-Stream
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Individual-Symbol-Ticker-Streams
+        """
+        return self._get("ticker", Ticker)
 
     @property
     def bookticker(self) -> "BookTicker":
-        return self.get("bookticker", BookTicker)
+        """bookTicker stream.
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#individual-symbol-book-ticker-streams
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Individual-Symbol-Book-Ticker-Streams
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Individual-Symbol-Book-Ticker-Streams
+        """
+        return self._get("bookticker", BookTicker)
 
     @property
     def orderbook(self) -> "OrderBook":
-        return self.get("orderbook", OrderBook)
+        """depthUpdate stream.
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/web-socket-streams#diff-depth-stream
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Diff-Book-Depth-Streams
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Diff-Book-Depth-Streams
+        """
+        return self._get("orderbook", OrderBook)
 
     @property
     def order(self) -> "Order":
+        """executionReport/ORDER_TRADE_UPDATE from User Data Streams.
+
+        アクティブオーダーのみデータが格納されます。 キャンセル、約定済みなどは削除されます。
+
+        * Spot
+            * https://developers.binance.com/docs/binance-spot-api-docs/user-data-stream#order-update
+        * USDⓈ-M
+            * https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams/Event-Order-Update
+        * COIN-M
+            * https://developers.binance.com/docs/derivatives/coin-margined-futures/user-data-streams/Event-Order-Update
         """
-        アクティブオーダーのみ(約定・キャンセル済みは削除される)
-        """
-        return self.get("order", Order)
+        return self._get("order", Order)
 
 
 class BinanceFuturesDataStoreBase(BinanceDataStoreBase):
-    _BALANCE_INIT_ENDPOINT = None
-    _POSITION_INIT_ENDPOINT = None
+    """Binance 先物の DataStoreCollection ベースクラス"""
+
+    _BALANCE_INIT_ENDPOINT: str
+    _POSITION_INIT_ENDPOINT: str
 
     def _init(self) -> None:
         super()._init()
-        self.create("markprice", datastore_class=MarkPrice)
-        self.create("continuouskline", datastore_class=ContinuousKline)
-        self.create("liquidation", datastore_class=Liquidation)
-        self.create("balance", datastore_class=Balance)
-        self.create("position", datastore_class=Position)
+        self._create("markprice", datastore_class=MarkPrice)
+        self._create("continuouskline", datastore_class=ContinuousKline)
+        self._create("liquidation", datastore_class=Liquidation)
+        self._create("balance", datastore_class=Balance)
+        self._create("position", datastore_class=Position)
 
     def _initialize_hook(self, resp: aiohttp.ClientResponse, data: Any, endpoint: str):
         if self._is_target_endpoint(self._BALANCE_INIT_ENDPOINT, endpoint):
@@ -255,26 +330,53 @@ class BinanceFuturesDataStoreBase(BinanceDataStoreBase):
 
     @property
     def markprice(self) -> "MarkPrice":
-        return self.get("markprice", MarkPrice)
+        """markPriceUpdate stream.
+
+        * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Mark-Price-Stream
+        * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Mark-Price-Stream
+        """
+        return self._get("markprice", MarkPrice)
 
     @property
     def continuouskline(self) -> "ContinuousKline":
-        return self.get("continuouskline", ContinuousKline)
+        """continuous_kline stream.
+
+        * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Continuous-Contract-Kline-Candlestick-Streams
+        * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Continuous-Contract-Kline-Candlestick-Streams
+        """
+        return self._get("continuouskline", ContinuousKline)
 
     @property
     def liquidation(self) -> "Liquidation":
-        return self.get("liquidation", Liquidation)
+        """forceOrder stream.
+
+        * https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Liquidation-Order-Streams
+        * https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Liquidation-Order-Streams
+        """
+        return self._get("liquidation", Liquidation)
 
     @property
     def balance(self) -> "Balance":
-        return self.get("balance", Balance)
+        """ACCOUNT_UPDATE from User Data Streams.
+
+        * https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams/Event-Balance-and-Position-Update
+        * https://developers.binance.com/docs/derivatives/coin-margined-futures/user-data-streams/Event-Balance-and-Position-Update
+        """
+        return self._get("balance", Balance)
 
     @property
     def position(self) -> "Position":
-        return self.get("position", Position)
+        """ACCOUNT_UPDATE from User Data Streams.
+
+        * https://developers.binance.com/docs/derivatives/usds-margined-futures/user-data-streams/Event-Balance-and-Position-Update
+        * https://developers.binance.com/docs/derivatives/coin-margined-futures/user-data-streams/Event-Balance-and-Position-Update
+        """
+        return self._get("position", Position)
 
 
 class BinanceSpotDataStore(BinanceDataStoreBase):
+    """Binance Spot の DataStoreCollection クラス"""
+
     _ORDERBOOK_INIT_ENDPOINT = "/api/v3/depth"
     _ORDER_INIT_ENDPOINT = "/api/v3/openOrders"
     _LISTENKEY_INIT_ENDPOINT = "/api/v3/userDataStream"
@@ -284,8 +386,8 @@ class BinanceSpotDataStore(BinanceDataStoreBase):
 
     def _init(self):
         super()._init()
-        self.create("account", datastore_class=Account)
-        self.create("ocoorder", datastore_class=OCOOrder)
+        self._create("account", datastore_class=Account)
+        self._create("ocoorder", datastore_class=OCOOrder)
 
     def _initialize_hook(self, resp: aiohttp.ClientResponse, data: Any, endpoint: str):
         if self._is_target_endpoint(self._ACCOUNT_INIT_ENDPOINT, endpoint):
@@ -312,14 +414,24 @@ class BinanceSpotDataStore(BinanceDataStoreBase):
 
     @property
     def account(self):
-        return self.get("account", Account)
+        """outboundAccountPosition from User Data Streams.
+
+        https://developers.binance.com/docs/binance-spot-api-docs/user-data-stream#account-update
+        """
+        return self._get("account", Account)
 
     @property
     def ocoorder(self):
-        return self.get("ocoorder", OCOOrder)
+        """listStatus from User Data Streams.
+
+        https://developers.binance.com/docs/binance-spot-api-docs/user-data-stream#conditional-fields-in-execution-report
+        """
+        return self._get("ocoorder", OCOOrder)
 
 
 class BinanceUSDSMDataStore(BinanceFuturesDataStoreBase):
+    """Binance USDⓈ-M の DataStoreCollection クラス"""
+
     _ORDERBOOK_INIT_ENDPOINT = "/fapi/v1/depth"
     _BALANCE_INIT_ENDPOINT = "/fapi/v2/balance"
     _ORDER_INIT_ENDPOINT = "/fapi/v1/openOrders"
@@ -330,7 +442,7 @@ class BinanceUSDSMDataStore(BinanceFuturesDataStoreBase):
 
     def _init(self):
         super()._init()
-        self.create("compositeindex", datastore_class=CompositeIndex)
+        self._create("compositeindex", datastore_class=CompositeIndex)
 
     def _initialize_hook(self, resp: aiohttp.ClientResponse, data: Any, endpoint: str):
         super()._initialize_hook(resp, data, endpoint)
@@ -347,10 +459,16 @@ class BinanceUSDSMDataStore(BinanceFuturesDataStoreBase):
 
     @property
     def compositeindex(self) -> "CompositeIndex":
-        return self.get("compositeindex", CompositeIndex)
+        """compositeindex stream.
+
+        https://developers.binance.com/docs/derivatives/usds-margined-futures/websocket-market-streams/Composite-Index-Symbol-Information-Streams
+        """
+        return self._get("compositeindex", CompositeIndex)
 
 
 class BinanceCOINMDataStore(BinanceFuturesDataStoreBase):
+    """Binance COIN-M の DataStoreCollection クラス"""
+
     _ORDERBOOK_INIT_ENDPOINT = "/dapi/v1/depth"
     _BALANCE_INIT_ENDPOINT = "/dapi/v1/balance"
     _ORDER_INIT_ENDPOINT = "/dapi/v1/openOrders"
@@ -364,9 +482,9 @@ class BinanceCOINMDataStore(BinanceFuturesDataStoreBase):
 
     def _init(self):
         super()._init()
-        self.create("indexprice", datastore_class=IndexPrice)
-        self.create("indexpricekline", datastore_class=Kline)
-        self.create("markpricekline", datastore_class=Kline)
+        self._create("indexprice", datastore_class=IndexPrice)
+        self._create("indexpricekline", datastore_class=Kline)
+        self._create("markpricekline", datastore_class=Kline)
 
     def _onmessage_hook(self, msg: Any, event: str, data: Any):
         super()._onmessage_hook(msg, event, data)
@@ -405,15 +523,27 @@ class BinanceCOINMDataStore(BinanceFuturesDataStoreBase):
 
     @property
     def indexprice(self) -> "IndexPrice":
-        return self.get("indexprice", IndexPrice)
+        """indexprice stream.
+
+        https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Index-Price-Stream
+        """
+        return self._get("indexprice", IndexPrice)
 
     @property
     def indexpricekline(self) -> "Kline":
-        return self.get("indexpricekline", Kline)
+        """indexpricekline stream.
+
+        https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Index-Kline-Candlestick-Streams
+        """
+        return self._get("indexpricekline", Kline)
 
     @property
     def markpricekline(self) -> "Kline":
-        return self.get("markpricekline", Kline)
+        """markpricekline stream.
+
+        https://developers.binance.com/docs/derivatives/coin-margined-futures/websocket-market-streams/Mark-Price-Kline-Candlestick-Streams
+        """
+        return self._get("markpricekline", Kline)
 
 
 class Trade(DataStore):
@@ -426,7 +556,7 @@ class Trade(DataStore):
 class MarkPrice(DataStore):
     _KEYS = ["s"]
 
-    def _onmessage(self, data: Union[Item, list[Item]]) -> None:
+    def _onmessage(self, data: Item | list[Item]) -> None:
         if isinstance(data, list):
             self._update(data)
         else:
@@ -511,7 +641,7 @@ class ContinuousKline(DataStore):
 class Ticker(DataStore):
     _KEYS = ["s"]
 
-    def _onmessage(self, data: Union[Item, list[Item]]) -> None:
+    def _onmessage(self, data: Item | list[Item]) -> None:
         if isinstance(data, list):
             self._update(data)
         else:
@@ -532,43 +662,47 @@ class Liquidation(DataStore):
 
 class OrderBook(DataStore):
     _KEYS = ["s", "S", "p"]
-    _MAPSIDE = {"BUY": "b", "SELL": "a"}
 
     def _init(self) -> None:
-        self.initialized = False
-        self._buff = deque(maxlen=200)
+        self.initialized: defaultdict[str, bool] = defaultdict(lambda: False)
+        self._buff: defaultdict[str, deque[Item]] = defaultdict(
+            lambda: deque(maxlen=8000)
+        )
 
-    def sorted(self, query: Optional[Item] = None) -> dict[str, list[float]]:
-        if query is None:
-            query = {}
-        result = {self._MAPSIDE[k]: [] for k in self._MAPSIDE}
-        for item in self:
-            if all(k in item and query[k] == item[k] for k in query):
-                result[self._MAPSIDE[item["S"]]].append([item["p"], item["q"]])
-        result["b"].sort(key=lambda x: float(x[0]), reverse=True)
-        result["a"].sort(key=lambda x: float(x[0]))
-        return result
+    def sorted(
+        self, query: Item | None = None, limit: int | None = None
+    ) -> dict[str, list[Item]]:
+        return self._sorted(
+            item_key="S",
+            item_asc_key="a",
+            item_desc_key="b",
+            sort_key="p",
+            query=query,
+            limit=limit,
+        )
 
     def _onmessage(self, item: Item) -> None:
-        if not self.initialized:
-            self._buff.append(item)
-        for s, bs in self._MAPSIDE.items():
-            for row in item[bs]:
+        if not self.initialized[item["s"]]:
+            self._buff[item["s"]].append(item)
+        for side in ("a", "b"):
+            for row in item[side]:
                 if float(row[1]) != 0.0:
-                    self._update([{"s": item["s"], "S": s, "p": row[0], "q": row[1]}])
+                    self._update(
+                        [{"s": item["s"], "S": side, "p": row[0], "q": row[1]}]
+                    )
                 else:
-                    self._delete([{"s": item["s"], "S": s, "p": row[0]}])
+                    self._delete([{"s": item["s"], "S": side, "p": row[0]}])
 
     def _onresponse(self, symbol: str, item: Item) -> None:
-        self.initialized = True
-        self._delete(self.find({"s": symbol}))
-        for s, bs in (("BUY", "bids"), ("SELL", "asks")):
-            for row in item[bs]:
-                self._insert([{"s": symbol, "S": s, "p": row[0], "q": row[1]}])
-        for msg in self._buff:
+        self._delete(self._find_and_delete({"s": symbol}))
+        for side_ws, side_http in (("a", "asks"), ("b", "bids")):
+            for row in item[side_http]:
+                self._insert([{"s": symbol, "S": side_ws, "p": row[0], "q": row[1]}])
+        for msg in self._buff[symbol]:
             if msg["U"] <= item["lastUpdateId"] and msg["u"] >= item["lastUpdateId"]:
                 self._onmessage(msg)
-        self._buff.clear()
+        self._buff[symbol].clear()
+        self.initialized[symbol] = True
 
 
 class Account(DataStore):
@@ -577,7 +711,7 @@ class Account(DataStore):
     def _onmessage(self, item: Item) -> None:
         self._update(item["B"])
 
-    def _onresponse(self, data: list[Item]):
+    def _onresponse(self, data: dict[str, list[Item]]):
         for item in data["balances"]:
             self._update(
                 [
@@ -652,13 +786,12 @@ class Order(DataStore):
         else:
             self._delete(items)
 
-    def _onresponse(self, symbol: Optional[str], data: list[Item]) -> None:
+    def _onresponse(self, symbol: str | None, data: list[Item]) -> None:
         if symbol is not None:
             self._delete(self.find({"s": symbol}))
         else:
             self._clear()
         for item in data:
-
             if "positionSide" in item:
                 # futures
                 self._insert(
